use strict;
use Module::Build;
my ($build, $params);


$params = {
            'license' => 'perl',
            'dist_abstract' => 'Date::EzDate2 - Date and time manipulation made easy',
            'dist_author' => 'Miko O\'Sullivan <miko@idocs.com>',
            'dist_version_from' => 'lib/Date/EzDate2.pm',
            'requires' => {
                            'Test::Most' => '0.34',
                            'Clone' => '0.38',
                            'DateTime::TimeZone' => '2.01',
                            'DateTime' => '1.34'
                          },
            'module_name' => 'Date::EzDate2'
          };


$build = Module::Build->new(%$params);
$build->create_build_script();
